"""This module is for the translation mapping data of the
following document:

Document file: event_prevent_default_and_stop_propagation.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Event class prevent_default and stop_propagation interfaces": "# Event クラスの prevent_default と stop_propagation のインターフェイス",  # noqa
    ##################################################
    "This page explains the `Event` class `prevent_default` and `stop_propagation` method interfaces.": "このページでは`Event`クラスの`prevent_default`と`stop_propagation`メソッドの各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `prevent_default` method interface appends the calling expression of the JavaScript `preventDefault` method. This interface prevents the browser's default behavior of any event.": "`prevent_default`メソッドのインターフェイスはJavaScriptの`preventDefault`メソッドに該当するコード表現を加えます。このインターフェイスはイベントにおけるブラウザのデフォルトの挙動を無効化します。",  # noqa
    ##################################################
    "The `stop_propagation` method interface stops an event's propagation; for example, the triggered child event does not propagate to a parent event (it ignores the parent event).": "`stop_propagation`メソッドのインターフェイスはイベントの伝搬を停止します。例えば、このインスタンス上で実行（発火）されたイベントは親のインスタンスへは伝搬しなくなります（親のイベントは無視されるようになります）。",  # noqa
    ##################################################
    "## Basic usage of the prevent_default interface": "## prevent_default インターフェイスの基本的な使い方",  # noqa
    ##################################################
    "The `Event`'s subclass instance has the `prevent_default` method (note: there is a subclass that does not have this interface). The `prevent_default` method requires no arguments, as follows:": "`Event`のサブクラスのインスタンスは`prevent_default`メソッドを持っています（注: このインターフェイスを持っていないサブクラスも存在します）。`prevent_default`メソッドは特に引数を必要としません。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.prevent_default()\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="event_prevent_default_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.prevent_default()\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="event_prevent_default_basic_usage/")\n```',  # noqa
    ##################################################
    "## Basic usage of the stop_propagation interface": "## stop_propagation インターフェイスの基本的な使い方",  # noqa
    ##################################################
    "The `Event`'s subclass instance has the `stop_propagation` method (note: there is a subclass that does not have this method). The `stop_propagation` method, like the `prevent_default` one, requires no arguments.": "`Event`のサブクラスのインスタンスは`stop_propagation`メソッドを持っています（注: このメソッドを持っていないサブクラスも存在します）。`stop_propagation`メソッドは`prevent_default`メソッドと同様に引数を必要としません。",  # noqa
    ##################################################
    "The following example binds the click event to the sprite and rectangle instances. The rectangle (child) click handler calls the `stop_propagation` method, so the sprite (parent) doesn't call the click handler:": "以下のコード例ではクリックイベントをSpriteの親のインスタンスと四角の子のインスタンスにそれぞれ設定しています。四角の子のインスタンスのクリックのハンドラでは`stop_propagation`メソッドを読んでいるため、親のSpriteのハンドラは呼ばれません（イベントが伝搬しません）。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_rectangle_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.stop_propagation()\n    ap.trace("The rectangle is clicked!")\n\n\ndef on_sprite_click(e: ap.MouseEvent[ap.Sprite], options: dict) -> None:\n    """\n    The handler that the sprite calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace("The sprite is clicked!")\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.click(on_sprite_click)\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_rectangle_click)\n\nap.save_overall_html(dest_dir_path="event_stop_propagation_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_rectangle_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.stop_propagation()\n    ap.trace("The rectangle is clicked!")\n\n\ndef on_sprite_click(e: ap.MouseEvent[ap.Sprite], options: dict) -> None:\n    """\n    The handler that the sprite calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace("The sprite is clicked!")\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.click(on_sprite_click)\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_rectangle_click)\n\nap.save_overall_html(dest_dir_path="event_stop_propagation_basic_usage/")\n```',  # noqa
    ##################################################
    "If you click the following rectangle, the only message of `The rectangle is clicked!` is displayed browser console (please press the F12 key). Also, the sprite console message is not displayed.": "もし以下の四角をクリックした場合、ブラウザのコンソールには`The rectangle is clicked!`というメッセージのみが表示され、Sprite関係のメッセージは表示されません。",  # noqa
    ##################################################
    "## prevent_default API": "## prevent_default API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Prevent event's default behavior.<hr>": "イベントのデフォルトの挙動を無効化します。<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     e.prevent_default()\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String("#f0a")\n...     rectangle.unbind_mouseup_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.click(on_click)\n```': '```py\n>>> import apysc as ap\n>>> def on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     e.prevent_default()\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String("#f0a")\n...     rectangle.unbind_mouseup_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.click(on_click)\n```',  # noqa
    ##################################################
    "## stop_propagation API": "## stop_propagation API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Stop an event propagation.<hr>": "イベントの伝搬を停止させます。<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_click(e: ap.MouseEvent, options: dict) -> None:\n...     e.stop_propagation()\n...     ap.trace("Clicked!")\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = sprite.click(on_click)\n>>> _ = rectangle.click(on_click)\n```': '```py\n>>> import apysc as ap\n>>> def on_click(e: ap.MouseEvent, options: dict) -> None:\n...     e.stop_propagation()\n...     ap.trace("Clicked!")\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = sprite.click(on_click)\n>>> _ = rectangle.click(on_click)\n```',  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: else.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Else class": "# Else クラス",
    ##################################################
    "This page explains the `Else` class.": "このページでは`Else`クラスについて説明します。",
    ##################################################
    "Before reading on, maybe it is helpful to read the following page (the apysc uses the `Else` class for the same reason as each apysc data type):": "このページを読み進める前に以下のページを確認しておくと役に立つかもしれません（apyscでは`Else`クラスを以下のデータ型のケースと同じ理由で使用しています）:",  # noqa
    ##################################################
    "- [Why the apysc library doesn't use the Python built-in data type](why_apysc_doesnt_use_python_builtin_data_type.md)": "- [なぜapyscではPythonのビルトインのデータの型を使用していないのか](jp_why_apysc_doesnt_use_python_builtin_data_type.md)",  # noqa
    ##################################################
    "## What is the Else class?": "## Else クラスの概要",
    ##################################################
    "The `Else` class is the apysc branch instruction class. It behaves like the Python built-in `else` keyword.": "`Else`クラスはapyscの条件分岐の指定用のクラスです。このクラスはPythonビルトインの`else`キーワードと同じように動作します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `Else` requires using the `with` statement. The `Else` class statement is only acceptable to implement right after the `If` or `Elif` classes statement.": "`Else`クラスは`with`ステートメントとセットで使用する必要があります。`Else`クラスのステートメントは`If`もしくは`Elif`クラスのステートメントの直後でのみ使用することができます。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\ncondition: ap.Boolean = ap.Boolean(False)\nint_1: ap.Int = ap.Int(10)\n\nwith ap.If(condition):\n    int_1 += 10\nwith ap.Else():\n    int_1 += 20\n```": "```py\n# runnable\nimport apysc as ap\n\ncondition: ap.Boolean = ap.Boolean(False)\nint_1: ap.Int = ap.Int(10)\n\nwith ap.If(condition):\n    int_1 += 10\nwith ap.Else():\n    int_1 += 20\n```",  # noqa
    ##################################################
    "## Notes": "## 特記事項",
    ##################################################
    "If you insert the code between the `If` (or `Elif`) and `Else` statements, it raises an exception:": "もしも`If`もしくは`Elif`クラスと`Else`クラスのステートメントの間にコードを挿入するとエラーとなります:",  # noqa
    ##################################################
    "```py\nimport apysc as ap\n\ncondition: ap.Boolean = ap.Boolean(False)\nint_1: ap.Int = ap.Int(10)\n\nwith ap.If(condition):\n    int_1 += 10\n# If there is a code implementation between the `If` and `Else`, then\n# exceptions will be raised.\nint_2: ap.Int = ap.Int(20)\nwith ap.Else():\n    int_1 += 20\n```": "```py\nimport apysc as ap\n\ncondition: ap.Boolean = ap.Boolean(False)\nint_1: ap.Int = ap.Int(10)\n\nwith ap.If(condition):\n    int_1 += 10\n# If there is a code implementation between the `If` and `Else`, then\n# exceptions will be raised.\nint_2: ap.Int = ap.Int(20)\nwith ap.Else():\n    int_1 += 20\n```",  # noqa
    ##################################################
    "```\nValueError: Else interface can only use right after If or Elif interfaces.\n```": "```\nValueError: Else interface can only use right after If or Elif interfaces.\n```",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [If class](if.md)": "- [If クラス](jp_if.md)",
    ##################################################
    "- [Elif class](elif.md)": "- [Elif クラス](jp_elif.md)",
    ##################################################
    "- [Each branch instruction class's scope variables reverting setting](branch_instruction_variables_reverting_setting.md)": "- [分岐条件の各クラスのスコープ内変数の復元設定](jp_branch_instruction_variables_reverting_setting.md)",  # noqa
    ##################################################
    "## Else constructor API": "## Else クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "A class to append else branch instruction expression.<hr>": "elseの分岐条件の表現を追加するためのクラスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `locals_`: dict or None, default None": "- `locals_`: dict or None, default None",  # noqa
    ##################################################
    "  - Current scope's local variables. Set locals() value to this argument. If specified, this interface reverts all local scope VariableNameMixIn variables (like Int, Sprite) at the end of an `Else` scope. This setting is useful when you don't want to update each variable by implementing the `Else` scope.": "  - 現在のスコープの各ローカル変数の値。利用する場合にはlocals()関数をこの引数へ指定してください。もし設定された場合には`Else`のスコープの最後にVariableNameMixInクラスを継承したクラスの各変数（例 : Spriteなど）の設定は復元されます。この設定は`Else`クラスのスコープ内で変数を更新したくない場合などに便利なことがあります。",  # noqa
    ##################################################
    "- `globals_`: dict or None, default None": "- `globals_`: dict or None, default None",  # noqa
    ##################################################
    "  - Current scope's global variables. Set globals() value to this argument. This setting works the same way as the locals_ argument.": "  - 現在のスコープの各グローバル変数。設定する場合にはglobal()関数の値をこの引数に指定してください。この設定はlocals_引数と同じように動作します。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    " ・You can only use this class immediately after the `If` or `Elif` statement.<hr>": " ・このクラスは`If`もしくは`Elif`クラスのステートメントの直後にのみ使用することができます。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> with ap.If(int_val >= 11):\n...     ap.trace("Value is greater than equal 11.")\n...\n>>> with ap.Else():\n...     ap.trace("Value is less than 11.")\n...\n```': '```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> with ap.If(int_val >= 11):\n...     ap.trace("Value is greater than equal 11.")\n...\n>>> with ap.Else():\n...     ap.trace("Value is less than 11.")\n...\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [Each branch instruction class's scope variables reverting setting](https://simon-ritchie.github.io/apysc/en/branch_instruction_variables_reverting_setting.html)": "- [分岐条件の各クラスのスコープ内変数の復元設定](https://simon-ritchie.github.io/apysc/jp/jp_branch_instruction_variables_reverting_setting.html)",  # noqa
}



"""Class implementation for the dictionary value.
"""

from typing import Any
from typing import Dict
from typing import Generic
from typing import TypeVar
from typing import Union

from typing_extensions import final

from apysc._event.custom_event_mixin import CustomEventMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.any_value import AnyValue
from apysc._type.boolean import Boolean
from apysc._type.copy_mixin import CopyMixIn
from apysc._type.dictionary_structure import DictionaryStructure
from apysc._type.expression_string import ExpressionString
from apysc._type.initial_substitution_exp_mixin import InitialSubstitutionExpMixIn
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.string import String
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos

DefaultType = TypeVar("DefaultType")

_BuiltinKeys = Union[str, int, float]
_K = TypeVar("_K")
_V = TypeVar("_V")


class Dictionary(
    CustomEventMixIn,
    Generic[_K, _V],
    CopyMixIn,
    RevertMixIn,
    DictionaryStructure,
    VariableNameSuffixMixIn,
    InitialSubstitutionExpMixIn,
):
    """
    Dictionary class for the apysc library.

    References
    ----------
    - Dictionary
        - https://simon-ritchie.github.io/apysc/en/dictionary.html
    - Dictionary class generic type settings
        - https://simon-ritchie.github.io/apysc/en/dictionary_generic.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> dictionary: ap.Dictionary = ap.Dictionary({"a": 10})
    >>> dictionary
    Dictionary({'a': 10})

    >>> dictionary["a"]
    10

    >>> dictionary["b"] = 20
    >>> dictionary["b"]
    20

    >>> dictionary.length
    Int(2)

    >>> value_1: int = dictionary.get("c", default=0)
    >>> value_1
    0
    """

    _initial_value: Union[Dict[_K, _V], "Dictionary"]
    _value: Dict[_K, _V]

    @arg_validation_decos.is_acceptable_dictionary_value(arg_position_index=1)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        value: Union[Dict[_K, _V], "Dictionary"],
        *,
        variable_name_suffix: str = "",
        skip_init_substitution_expression_appending: bool = False,
    ) -> None:
        """
        Dictionary class for the apysc library.

        Parameters
        ----------
        value : dict or Dictionary
            Initial dictionary value.
        variable_name_suffix : str, default ''
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript's debugging.
        skip_init_substitution_expression_appending : bool, default False
            A boolean indicates whether to skip an initial substitution
            expression or not. This class uses this option internally.

        References
        ----------
        - Dictionary
            - https://simon-ritchie.github.io/apysc/en/dictionary.html
        - Dictionary class generic type settings
            - https://simon-ritchie.github.io/apysc/en/dictionary_generic.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> dictionary: ap.Dictionary = ap.Dictionary({"a": 10})
        >>> dictionary
        Dictionary({'a': 10})

        >>> dictionary["a"]
        10

        >>> dictionary["b"] = 20
        >>> dictionary["b"]
        20
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._expression.event_handler_scope import TemporaryNotHandlerScope

        with TemporaryNotHandlerScope():
            self._variable_name_suffix = variable_name_suffix
            TYPE_NAME: str = var_names.DICTIONARY
            self._initial_value = value
            self._type_name = TYPE_NAME
            self._value = self._get_dict_value(value=value)
            self.variable_name = expression_variables_util.get_next_variable_name(
                type_name=TYPE_NAME
            )
            self._append_constructor_expression()

        self._append_initial_substitution_expression_if_in_handler_scope(
            skip_appending=skip_init_substitution_expression_appending,
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_constructor_expression(self) -> None:
        """
        Append constructor expression.
        """
        import apysc as ap

        expression: str = self._create_initial_substitution_expression()
        expression = f"var {expression}"
        ap.append_js_expression(expression=expression)

    @final
    def _create_initial_substitution_expression(self) -> str:
        """
        Create an initial value's substitution expression string.

        Returns
        -------
        expression : str
            Created expression string.
        """
        from apysc._type import value_util

        value_str: str = value_util.get_value_str_for_expression(
            value=self._initial_value
        )
        expression: str = f"{self.variable_name} = {value_str};"
        return expression

    @final
    def _get_dict_value(
        self, *, value: Union[Dict[_K, _V], "Dictionary"]
    ) -> Dict[_K, _V]:
        """
        Get a dict value from a specified value.

        Parameters
        ----------
        value : dict or Dictionary
            Specified dictionary value.

        Returns
        -------
        dict_val : dict
            Converted dict value.
        """
        if isinstance(value, Dictionary):
            return value._value
        return value  # type: ignore

    @property
    @add_debug_info_setting(module_name=__name__)
    def value(self) -> Union[Dict[_K, _V], "Dictionary"]:
        """
        Get a current dict value.

        Returns
        -------
        value : dict
            Current dict value.

        References
        ----------
        - apysc fundamental data classes value interface
            - https://simon-ritchie.github.io/apysc/en/fundamental_data_classes_value_interface.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> dictionary: ap.Dictionary = ap.Dictionary({})
        >>> dictionary.value = {"a": 10}
        >>> dictionary.value
        {'a': 10}
        """
        return self._value

    @value.setter
    @arg_validation_decos.is_acceptable_dictionary_value(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def value(self, value: Union[Dict[_K, _V], "Dictionary"]) -> None:
        """
        Set dictionary value.

        Parameters
        ----------
        value : dict or Dictionary.
            Dictionary value to set.

        References
        ----------
        - apysc fundamental data classes value interface
            - https://simon-ritchie.github.io/apysc/en/fundamental_data_classes_value_interface.html  # noqa
        """
        self._value = self._get_dict_value(value=value)
        self._append_value_setter_expression(value=value)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_value_setter_expression(
        self, *, value: Union[Dict[_K, _V], "Dictionary"]
    ) -> None:
        """
        Append value's setter expression.

        Parameters
        ----------
        value : dict or Dictionary.
            Dictionary value to set.
        """
        import apysc as ap
        from apysc._type import value_util

        value_str: str = value_util.get_value_str_for_expression(value=value)
        expression: str = f"{self.variable_name} = {value_str};"
        ap.append_js_expression(expression=expression)

    _value_snapshot: Dict[str, Dict[_K, _V]]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make values' snapthot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._set_single_snapshot_val_to_dict(
            dict_name="_value_snapshot",
            value={**self._value},
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert values if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._value = self._value_snapshot[snapshot_name]

    def __str__(self) -> str:
        """
        String conversion method.

        Returns
        -------
        string : str
            Converted value string.
        """
        if not hasattr(self, "_value"):
            return "{}"
        return str(self._value)

    def __repr__(self) -> str:
        """
        Get a representation string of this instance.

        Returns
        -------
        repr_str : str
            Representation string of this instance.
        """
        if not hasattr(self, "_value"):
            repr_str: str = "Dictionary({})"
        else:
            repr_str = f"Dictionary({self._value})"
        return repr_str

    @property
    @add_debug_info_setting(module_name=__name__)
    def length(self) -> Int:
        """
        Get length of this dictionary values.

        Returns
        -------
        length : Int
            This dictionary value's length.

        References
        ----------
        - Dictionary length interface
            - https://simon-ritchie.github.io/apysc/en/dictionary_length.html

        Examples
        --------
        >>> import apysc as ap
        >>> dictionary: ap.Dictionary = ap.Dictionary({"a": 1, "b": 2})
        >>> dictionary.length
        Int(2)
        """
        import apysc as ap

        length: ap.Int = ap.Int(len(self._value))
        self._append_length_expression(length=length)
        return length

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_length_expression(self, *, length: Int) -> None:
        """
        Append length method expression.

        Parameters
        ----------
        length : Int
            Created length Int variable.
        """
        import apysc as ap

        expression: str = (
            f"{length.variable_name} = " f"Object.keys({self.variable_name}).length;"
        )
        ap.append_js_expression(expression=expression)

    @final
    def __len__(self) -> None:
        """
        This method is disabled and can't use from a
        Dictionary instance.
        """
        raise Exception(
            "Dictionary instance can't apply len function."
            " Please use length property instead."
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def __getitem__(self, key: Union[_K, ExpressionString]) -> _V:
        """
        Get a specified key's single value.

        Parameters
        ----------
        key : _K
            Dictionary key.

        Returns
        -------
        value : *
            Specified key's value.
        """
        import apysc as ap

        self._validate_key_type_is_str_or_numeric(key=key)
        key_: _BuiltinKeys = self._get_builtin_type_key(key=key)
        has_key: bool = key_ in self._value
        if has_key:
            value: Any = self._value[key_]  # type: ignore
        else:
            value = ap.AnyValue(None)
        self._append_getitem_expression(key=key, value=value)
        return value

    @final
    def _get_builtin_type_key(
        self, *, key: Union[_K, ExpressionString]
    ) -> _BuiltinKeys:
        """
        Get a built-in type's key (str, int, or float) from
        a specified key.

        Parameters
        ----------
        key : _K
            Target key value (including String, Int, and Number).

        Returns
        -------
        key : str or int or float
            Built-in type's key.
        """
        if isinstance(key, ExpressionString):
            return key.value
        if isinstance(key, (String, Int, Number)):
            return key._value
        key_: _BuiltinKeys = key  # type: ignore
        return key_

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_getitem_expression(
        self, *, key: Union[_K, ExpressionString], value: Any
    ) -> None:
        """
        Append __getitem__ expression.

        Parameters
        ----------
        key : _K
            Dictionary key.
        value : *
            Specified key's value.
        """
        import apysc as ap
        from apysc._type import value_util

        if not isinstance(value, VariableNameMixIn):
            value = ap.AnyValue(None)
        key_str: str = value_util.get_value_str_for_expression(value=key)
        expression: str = (
            f"var {value.variable_name} = " f"{self.variable_name}[{key_str}];"
        )
        ap.append_js_expression(expression=expression)

    @final
    def _validate_key_type_is_str_or_numeric(
        self, *, key: Union[_K, ExpressionString]
    ) -> None:
        """
        Validate whether a key's value type is acceptable
        (str or int or float) or not.

        Parameters
        ----------
        key : _K
            Dictionary key to check.

        Raises
        ------
        ValueError
            If a key's type is not String, Int, Number, str, int,
            or float.
        """
        if isinstance(key, (str, String, int, Int, float, Number, ExpressionString)):
            return
        raise ValueError(
            f"Unsupported key type is specified: {type(key)}, {key}"
            f"\nSuppoting types are: str, String, int, Int"
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def __setitem__(self, key: Union[_K, ExpressionString], value: _V) -> None:
        """
        Set value to a specified key position.

        Parameters
        ----------
        key : _K
            Dictionary key to set value.
        value : *
            Any value to set.
        """
        key_: _BuiltinKeys = self._get_builtin_type_key(key=key)
        self._value[key_] = value  # type: ignore
        self._append_setitem_expression(key=key, value=value)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_setitem_expression(
        self, *, key: Union[_K, ExpressionString], value: _V
    ) -> None:
        """
        Append __setitem__ method expression.

        Parameters
        ----------
        key : _K
            Dictionary key to check.
        value : *
            Any value to set.
        """
        import apysc as ap
        from apysc._type import value_util

        key_str: str = value_util.get_value_str_for_expression(value=key)
        value_str: str = value_util.get_value_str_for_expression(value=value)
        expression: str = f"{self.variable_name}[{key_str}] = {value_str};"
        ap.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def __delitem__(self, key: Union[_K, ExpressionString]) -> None:
        """
        Delete specified key's value from a dictionary.

        Parameters
        ----------
        key : _K
            Dictionary key to delete.
        """
        key_: _BuiltinKeys = self._get_builtin_type_key(key=key)
        if key_ in self._value:
            del self._value[key_]  # type: ignore
        self._append_delitem_expression(key=key)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_delitem_expression(self, *, key: Union[_K, ExpressionString]) -> None:
        """
        Append __delitem__ method expression.

        Parameters
        ----------
        key : _K
            Dictionary key to delete.
        """
        import apysc as ap
        from apysc._type import value_util

        key_str: str = value_util.get_value_str_for_expression(value=key)
        expression: str = f"delete {self.variable_name}[{key_str}];"
        ap.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def __eq__(self, other: Any) -> Any:
        """
        Equal comparison method.

        Parameters
        ----------
        other : *
            The other value to compare. dict or Dictionary
            types are acceptable.

        Returns
        -------
        result : Boolean
            Comparison result.
        """
        import apysc as ap

        if isinstance(other, Dictionary):
            result: ap.Boolean = ap.Boolean(self._value == other._value)
        else:
            result = ap.Boolean(self._value == other)
            if isinstance(other, dict):
                other = Dictionary(other)
        if isinstance(other, VariableNameMixIn):
            self._append_eq_expression(result=result, other=other)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_eq_expression(
        self, *, result: Boolean, other: VariableNameMixIn
    ) -> None:
        """
        Append an __eq__ expression.

        Parameters
        ----------
        result : Boolean
            A result boolean value.
        other : Dictionary
            The Dictionary's other value to compare.
        """
        import apysc as ap

        expression: str = (
            f"{result.variable_name} = "
            f"_.isEqual({self.variable_name}, {other.variable_name});"
        )
        ap.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def __ne__(self, other: Any) -> Any:
        """
        Noe equal comparison method.

        Parameters
        ----------
        other : *
            The other value to compare. dict or Dictionary types
            are acceptable.

        Returns
        -------
        result : Boolean
            Comparison result.
        """
        import apysc as ap

        if isinstance(other, dict):
            other = Dictionary(other)
        result: ap.Boolean = self == other
        result = result.not_
        if isinstance(other, VariableNameMixIn):
            self._append_ne_expression(result=result, other=other)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_ne_expression(
        self, *, result: Boolean, other: VariableNameMixIn
    ) -> None:
        """
        Append a __ne__ expression.

        Parameters
        ----------
        result : Boolean
            A result boolean value.
        other : Dictionary
            The Dictionary's other value to compare.
        """
        import apysc as ap

        expression: str = (
            f"{result.variable_name} = "
            f"!_.isEqual({self.variable_name}, {other.variable_name});"
        )
        ap.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def get(
        self, key: Union[_K, ExpressionString], *, default: DefaultType = None
    ) -> DefaultType:
        """
        Get a specified key dictionary value. If this
        dictionary hasn't a specified key, this interface
        returns a default value.

        Parameters
        ----------
        key : _K
            Target key.
        default : DefaultType or None, optional
            Any default value.

        Returns
        -------
        result_value : Any
            Extracted value or a default value.

        References
        ----------
        - Dictionary get interface
            - https://simon-ritchie.github.io/apysc/en/dictionary_get.html

        Examples
        --------
        >>> from typing import Optional
        >>> import apysc as ap
        >>> dictionary: ap.Dictionary = ap.Dictionary({"a": 10})
        >>> value_1: Optional[int] = dictionary.get("a")
        >>> value_1
        10

        >>> value_2: Optional[int] = dictionary.get("b")
        >>> print(value_2)
        None

        >>> value_3: int = dictionary.get("c", default=0)
        >>> value_3
        0
        """
        key_: _BuiltinKeys = self._get_builtin_type_key(key=key)
        if key_ in self._value:
            result_value: Any = self._value[key_]  # type: ignore
        else:
            result_value = default
        self._append_get_expression(key=key, result_value=result_value, default=default)
        return result_value

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_get_expression(
        self, *, key: Union[_K, ExpressionString], result_value: Any, default: Any
    ) -> None:
        """
        Append the `get` method expression.

        Parameters
        ----------
        key : _K
            Target key.
        result_value : Any
            Extracted value or a default value.
        default : Any
            Any default value. Basic apysc types (e.g.,
            Int, Number, String, and so on) are necessary.
        """
        import apysc as ap
        from apysc._type import value_util

        key_str: str = value_util.get_value_str_for_expression(value=key)
        if not isinstance(result_value, VariableNameMixIn):
            result_value = AnyValue(value=None)
        result_value_str: str = value_util.get_value_str_for_expression(
            value=result_value
        )
        default_value_str: str = value_util.get_value_str_for_expression(value=default)
        expression: str = (
            f"if ({key_str} in {self.variable_name}) {{"
            f"\n  {result_value_str} = {self.variable_name}[{key_str}];"
            "\n}else {"
            f"\n  {result_value_str} = {default_value_str};"
            "\n}"
        )
        ap.append_js_expression(expression=expression)



"""Mix-in class implementation for the y path data.
"""

from typing import Dict

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.int import Int
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_suffix_attr_mixin import VariableNameSuffixAttrMixIn
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class PathYMixIn(
    VariableNameSuffixAttrMixIn,
    RevertMixIn,
    AttrLinkingMixIn,
    VariableNameSuffixMixIn,
):

    _y: Int

    @final
    def _initialize_y_if_not_initialized(self) -> None:
        """
        Initialize the _y attribute if this instance does not
        initialize it yet.
        """
        if hasattr(self, "_y"):
            return
        suffix: str = self._get_attr_variable_name_suffix(attr_identifier="y")
        self._y = Int(
            0,
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

        self._append_y_linking_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_y_linking_setting(self) -> None:
        """
        Append a y attribute linking settings.
        """
        self._append_applying_new_attr_val_exp(new_attr=self._y, attr_name="y")
        self._append_attr_to_linking_stack(attr=self._y, attr_name="y")

    @property
    @add_debug_info_setting(module_name=__name__)
    def y(self) -> Int:
        """
        Get a y-coordinate of the destination point.

        Returns
        -------
        y : Int
            A y-coordinate of the destination point.

        Examples
        --------
        >>> import apysc as ap
        >>> line_to: ap.PathLineTo = ap.PathLineTo(x=50, y=50)
        >>> line_to.y = ap.Int(100)
        >>> line_to.y
        Int(100)
        """
        self._initialize_y_if_not_initialized()
        return self._y._copy()

    @y.setter
    @arg_validation_decos.is_apysc_num(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def y(self, value: Int) -> None:
        """
        Set a y-coordinate of the destination point.

        Parameters
        ----------
        value : Int
            Y-coordinate of the destination point.
        """
        self._initialize_y_if_not_initialized()
        self._y.value = value

        self._append_y_linking_setting()

    _y_snapshots: Dict[str, int]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_y_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_y_snapshots",
            value=int(self._y._value),
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._initialize_y_if_not_initialized()
        self._y._value = self._y_snapshots[snapshot_name]



"""This module is for the translation mapping data of the
following document:

Document file: draw_interfaces_abstract.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Draw interfaces abstract": "# 描画の各インターフェイスの概要",
    ##################################################
    "This page would explain the drawing interfaces abstract.": "このページでは描画の各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What apysc can do in its drawing interfaces": "## apyscの描画の各インターフェイスでできること",
    ##################################################
    "- You can set the fill-color, fill-alpha (opacity), line-color, line-alpha, line-thickness values and draw each SVG graphic with these.": "- これらのインターフェイスを使って塗りの色、塗りの透明度、線の色、線の透明度、線幅などを設定することができます。",  # noqa
    ##################################################
    "- Supported graphics: The rectangle, circle, ellipse, polygon, line, polyline, and path.": "- 四角や丸、楕円、多角形、線、折れ線、パスなどの描画をサポートしています。",  # noqa
    ##################################################
    "## Fill settings": "## 塗りの設定",
    ##################################################
    "The `begin_fill` interface sets the fill-color and fill-alpha (opacity).": "`begin_fill`のインターフェイスは塗りの色と塗りの透明度を設定します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af", alpha=0.5)\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nap.save_overall_html(dest_dir_path="draw_interfaces_abstract_begin_fill/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af", alpha=0.5)\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nap.save_overall_html(dest_dir_path="draw_interfaces_abstract_begin_fill/")\n```',  # noqa
    ##################################################
    "For more details, please see:": "詳細は以下をご確認ください:",
    ##################################################
    "- [Graphics class begin_fill interface](graphics_begin_fill.md).": "- [Graphics クラスの begin_fill (塗りの設定)のインターフェイス](jp_graphics_begin_fill.md)",  # noqa
    ##################################################
    "## Line style settings": "## 線のスタイル設定",
    ##################################################
    "The `line_style` interface sets the line-color, line-alpha (opacity), line-thickness.": "`line_style`インターフェイスは線の色と線の透明度、線幅などを設定することができます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=200, stage_height=100, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color="#fff", thickness=5, alpha=0.5)\nsprite.graphics.draw_line(x_start=50, y_start=50, x_end=150, y_end=50)\n\nap.save_overall_html(dest_dir_path="draw_interfaces_abstract_line_style/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=200, stage_height=100, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color="#fff", thickness=5, alpha=0.5)\nsprite.graphics.draw_line(x_start=50, y_start=50, x_end=150, y_end=50)\n\nap.save_overall_html(dest_dir_path="draw_interfaces_abstract_line_style/")\n```',  # noqa
    ##################################################
    "For more details, please see:": "詳細は以下をご確認ください:",
    ##################################################
    "- [Graphics class line_style interface](graphics_line_style.md).": "- [Graphics クラスの line_style (線のスタイル設定)のインターフェイス](jp_graphics_line_style.md)",  # noqa
    ##################################################
    "## Each drawing interface": "## 描画の各インターフェイス",
    ##################################################
    "Each drawing interface has the `draw_` prefix draw SVG graphics (e.g., draw_rect, draw_circle).": "各描画のインターフェイスは`draw_`のプレフィックスを持っており、SVGのグラフィックを描画します（例 : draw_rect や draw_circle など）。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=250, stage_height=150, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nsprite.graphics.draw_circle(x=175, y=75, radius=25)\n\nap.save_overall_html(dest_dir_path="draw_interfaces_abstract_each_drawing_interface/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=250, stage_height=150, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nsprite.graphics.draw_circle(x=175, y=75, radius=25)\n\nap.save_overall_html(dest_dir_path="draw_interfaces_abstract_each_drawing_interface/")\n```',  # noqa
    ##################################################
    "For more details, please see the following:": "詳細については以下をご確認ください:",
    ##################################################
    "- [Graphics class draw_rect interface](graphics_draw_rect.md)": "- [Graphics クラスの draw_rect (四角の描画)のインターフェイス](jp_graphics_draw_rect.md)",  # noqa
    ##################################################
    "- [Graphics class draw_round_rect interface](graphics_draw_round_rect.md)": "- [Graphics クラスの draw_round_rect (角丸の四角の描画)のインターフェイス](jp_graphics_draw_round_rect.md)",  # noqa
    ##################################################
    "- [Graphics class draw_circle interface](graphics_draw_circle.md)": "- [Graphics クラスの draw_circle (円の描画)のインターフェイス](jp_graphics_draw_circle.md)",  # noqa
    ##################################################
    "- [Graphics class draw_ellipse interface](graphics_draw_ellipse.md)": "- [Graphics クラスの draw_ellipse (楕円描画) のインターフェイス](jp_graphics_draw_ellipse.md)",  # noqa
    ##################################################
    "- [Graphics class move_to and line_to interfaces](graphics_move_to_and_line_to.md)": "- [Graphics クラスの move_to (線の描画位置の変更)と line_to (指定座標への線の描画)のインターフェイス](jp_graphics_move_to_and_line_to.md)",  # noqa
    ##################################################
    "- [Graphics class draw_line interface](graphics_draw_line.md)": "- [Graphics クラスの draw_line (線の描画)のインターフェイス](jp_graphics_draw_line.md)",  # noqa
    ##################################################
    "- [Graphics class draw_dotted_line interface](graphics_draw_dotted_line.md)": "- [Graphics クラスの draw_dotted_line (点線の描画)のインターフェイス](jp_graphics_draw_dotted_line.md)",  # noqa
    ##################################################
    "- [Graphics class draw_dashed_line interface](graphics_draw_dashed_line.md)": "- [Graphics クラスの draw_dashed_line (破線の描画)のインターフェイス](jp_graphics_draw_dashed_line.md)",  # noqa
    ##################################################
    "- [Graphics class draw_round_dotted_line interface](graphics_draw_round_dotted_line.md)": "- [Graphics クラスの draw_round_dotted_line (点線(丸)の描画)のインターフェイス](jp_graphics_draw_round_dotted_line.md)",  # noqa
    ##################################################
    "- [Graphics class draw_dash_dotted_line interface](graphics_draw_dash_dotted_line.md)": "- [Graphics クラスの draw_dash_dotted_line (一点鎖線の描画)のインターフェイス](jp_graphics_draw_dash_dotted_line.md)",  # noqa
    ##################################################
    "- [Graphics class draw_polygon interface](graphics_draw_polygon.md)": "- [Graphics クラスの draw_polygon (多角形描画)のインターフェイス](jp_graphics_draw_polygon.md)",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [Graphics class](graphics.md)": "- [Graphics クラス](jp_graphics.md)",
    ##################################################
    "- [GraphicsBase class fill_color interface](graphics_base_fill_color.md)": "- [GraphicsBase クラスの fill_color (塗り設定)のインターフェイス](jp_graphics_base_fill_color.md)",  # noqa
    ##################################################
    "- [GraphicsBase class fill_alpha interface](graphics_base_fill_alpha.md)": "- [GraphicsBase クラスの fill_alpha (塗りの透明度設定)のインターフェイス](jp_graphics_base_fill_alpha.md)",  # noqa
    ##################################################
    "- [GraphicsBase class line_color interface](graphics_base_line_color.md)": "- [GraphicsBase クラスの line_color (線の色設定)のインターフェイス](jp_graphics_base_line_color.md)",  # noqa
    ##################################################
    "- [GraphicsBase class line_alpha interface](graphics_base_line_alpha.md)": "- [GraphicsBase クラスの line_color (線の透明度設定)のインターフェイス](jp_graphics_base_line_alpha.md)",  # noqa
    ##################################################
    "- [GraphicsBase class line_thickness interface](graphics_base_line_thickness.md)": "- [GraphicsBase クラスの line_color (線幅設定)のインターフェイス](jp_graphics_base_line_thickness.md)",  # noqa
    ##################################################
    "- [GraphicsBase class line_dot_setting interface](graphics_base_line_dot_setting.md)": "- [GraphicsBase クラスの line_dot_setting (点線設定)のインターフェイス](jp_graphics_base_line_dot_setting.md)",  # noqa
    ##################################################
    "- [GraphicsBase class line_dash_setting interface](graphics_base_line_dash_setting.md)": "- [GraphicsBase クラスの line_dash_setting (破線設定)のインターフェイス](jp_graphics_base_line_dash_setting.md)",  # noqa
    ##################################################
    "- [GraphicsBase class line_round_dot_setting interface](graphics_base_line_round_dot_setting.md)": "- [GraphicsBase クラスの line_round_dot_setting (点線(丸)設定)のインターフェイス](jp_graphics_base_line_round_dot_setting.md)",  # noqa
    ##################################################
    "- [GraphicsBase class line_dash_dot_setting interface](graphics_base_line_dash_dot_setting.md)": "- [GraphicsBase クラスの line_dash_dot_setting (一点鎖線設定)のインターフェイス](jp_graphics_base_line_dash_dot_setting.md)",  # noqa
}
